#include "StdAfx.h"
#include ".\bmsgenerator.hpp"
#include "BmsUtility.hpp"

namespace Lmt {
namespace Max2Bms {
	BmsGenerator::BmsGenerator( void ) {
	}

	BmsGenerator::~BmsGenerator( void ) {
	}

	BmsHeader * BmsGenerator::GetHeader( void ) {
		return &m_header;
	}

	BmsMaterial * BmsGenerator::GetMaterial( void ) {
		return &m_material;
	}

	BmsSequence * BmsGenerator::GetSequence( void ) {
		return &m_sequence;
	}

	int BmsGenerator::GetLineCount( void ) const {
		return static_cast<int>( m_lines.size() );
	}

	wstring BmsGenerator::GetLine( int index ) const {
		return m_lines[index];
	}

	void BmsGenerator::Clear( void ) {
		m_header.Clear();
		m_material.Clear();
		m_sequence.Clear();
		m_lines.clear();
	}

	void BmsGenerator::Generate( void ) {
		list<wstring> lines;
		list<wstring>::iterator itSrc;
		BmsLineArray::iterator itDest;

		this->BuildHeaderScript( lines );
		this->BuildMaterialScript( lines );
		this->BuildSequenceScript( lines );

		m_lines.clear();
		m_lines.resize( lines.size() );

		itSrc = lines.begin();
		itDest = m_lines.begin();
		for (; itSrc != lines.end(); itSrc++) {
			itDest->swap( *itSrc );
			itDest++;
		}
	}

	void BmsGenerator::BuildHeaderScript( list<wstring> &lines ) {
		wstringstream ss;

		lines.push_back( L";" );
		lines.push_back( L"; DJMAX - BMS" );
		lines.push_back( L";" );
		lines.push_back( L"; Created by LM-T PT2BMS ver0.01" );
		lines.push_back( L";" );
		lines.push_back( L"; Header-----" );
		lines.push_back( L";" );

		ss.str( L"" );
		ss << L"#PLAYER " << static_cast<int>( m_header.GetPlayStyle() );
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#GENRE " << m_header.GetGenre();
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#TITLE " << m_header.GetTitle();
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#ARTIST " << m_header.GetArtist();
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#BPM " << m_header.GetInitialTempo();
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#PLAYLEVEL " << m_header.GetDifficulty();
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#RANK " << static_cast<int>( m_header.GetJudgeRank() );
		lines.push_back( ss.str() );

		ss.str( L"" );
		ss << L"#TOTAL " << m_header.GetTotalGroove();
		lines.push_back( ss.str() );

		if (m_header.GetTitleImagePath().length() > 0) {
			ss.str( L"" );
			ss << L"#STAGEFILE " << m_header.GetTitleImagePath();
			lines.push_back( ss.str() );
		}

		lines.push_back( L"" );
	}

	void BmsGenerator::BuildMaterialScript( list<wstring> &lines ) {
		wstringstream ss;
		wstring path;

		lines.push_back( L";" );
		lines.push_back( L"; Materials-----" );
		lines.push_back( L";" );

		for (int i = 0; i < BmsMaterial::MaxMaterialCount; i++) {
			path = m_material.GetSoundPath( AlphaDecimal( i ) );

			if (path.length() > 0) {
				ss.str( L"" );
				ss << "#WAV" << AlphaDecimal( i ).ToString() << L" ";
				ss << path;

				lines.push_back( ss.str() );
			}
		}

		lines.push_back( L"" );
	}

	void BmsGenerator::BuildSequenceScript( list<wstring> &lines ) {
		static const BmsChannels channels[] = {
			P1Line1BmsCh, P1Line2BmsCh, P1Line3BmsCh, P1Line4BmsCh, P1Line5BmsCh, P1Line6BmsCh, P1Line7BmsCh, 
			P1Ln1BmsCh, P1Ln2BmsCh, P1Ln3BmsCh, P1Ln4BmsCh, P1Ln5BmsCh, P1Ln6BmsCh, P1Ln7BmsCh, 
			ChangeTempoBmsCh,
			static_cast<BmsChannels>( -1 ),
		};

		wstringstream ss;
		wstring measureStr;
		const BmsMeasure *pmeasure;
		int lastMeasureNo;
		int i;
		int j;

		lines.push_back( L";" );
		lines.push_back( L"; Sequence-----" );
		lines.push_back( L";" );

		lastMeasureNo = m_sequence.GetLastMeasureNo();

		for (i = 1; i <= lastMeasureNo; i++) {
			ss.str( L"" );
			ss << L";[" << i << L"]";
			lines.push_back( ss.str() );

			for (j = 0; channels[j] != -1; j++) {
				pmeasure = m_sequence.GetMeasure( channels[j], i );
				if (pmeasure == NULL) {
					continue;
				}

				if (channels[j] == ChangeTempoBmsCh) {
					measureStr = pmeasure->ToHexString();
				}
				else {
					measureStr = pmeasure->ToString();
				}

				if (measureStr.length() > 0) {
					ss.str( L"" );
					ss << BmsUtility::MakeSequenceHeader( i, channels[j] );
					ss << measureStr;
					lines.push_back( ss.str() );
				}
			}

			for (j = 0; j < BmsSequence::MaxBackgroundCount; j++) {
				m_sequence.SetBackgroundIndex( j );

				pmeasure = m_sequence.GetMeasure( BackgroundBmsCh, i );
				if (pmeasure == NULL) {
					continue;
				}

				measureStr = pmeasure->ToString();
				if (measureStr.length() > 0) {
					ss.str( L"" );
					ss << BmsUtility::MakeSequenceHeader( i, BackgroundBmsCh );
					ss << measureStr;
					lines.push_back( ss.str() );
				}
			}

			lines.push_back( L"" );
		}
	}
}
}
