#include "StdAfx.h"
#include ".\bmsmeasure.hpp"
#include "Utility.hpp"

namespace Lmt {
namespace Max2Bms {
	bool BmsMeasure::m_init = false;
	int BmsMeasure::m_gcds[Resolution];

	BmsMeasure::BmsMeasure( void ) {
		int i;

		if (m_init == false) {
			m_init = true;
			for (i = 0; i < Resolution; i++) {
				m_gcds[i] = Utility::GreatestCommonDivisor( Resolution, (i + 1) );
			}
		}

		for (i = 0; i < Resolution; i++) {
			m_values[i].Set( 0 );
		}
	}

	BmsMeasure::~BmsMeasure( void ) {
	}

	wstring BmsMeasure::ToString( void ) const {
		return this->ToStringInner( false );
	}

	wstring BmsMeasure::ToHexString( void ) const {
		return this->ToStringInner( true );
	}

	AlphaDecimal & BmsMeasure::operator []( int index ) {
		if ((index < 0) || (index >= Resolution)) {
			return m_values[Resolution];
		}
		return *(m_values + index);
	}

	int BmsMeasure::Optimize() const {
		static const struct CheckValues {
			int algorithm;  //0 = mod, 1 = and
			int checkValue;
			int stepCount;
		}
		checks[] = {
			{ 0, 96, 96 }, { 1, 0x2f, 64 }, { 0, 48, 48 }, { 1, 0x1f, 32 }, 
			{ 0, 24, 24 }, { 1, 0x0f, 16 }, { 0, 12, 12 }, { 1, 0x07, 8 }, 
			{ 0, 6, 6 }, { 1, 0x03, 4 },  { 0, 3, 3 }, { 1, 0x01, 1 },
			{ -1 },
		};

		int step = Resolution;
		int position;

		for (int i = 0; i < Resolution; i++) {
			if ((m_values[i].Get() != 0) && (i > 0)) {
				position = i;

				for (int j = 0; checks[j].algorithm != -1; j++) {
					if (step <= checks[j].stepCount) {
						break;
					}
					if ((checks[j].algorithm == 0) && ((position % checks[j].checkValue) != 0)) {
						step = checks[j].stepCount;
						break;
					}
					if ((checks[j].algorithm == 1) && ((position & checks[j].checkValue) != 0)) {
						step = checks[j].stepCount;
						break;
					}
				}
				if (step == 1) {
					break;
				}
			}
		}

		if ((step == Resolution) && (m_values[0].Get() == 0)) {
			step = 0;
		}

		return step;
	}

	wstring BmsMeasure::ToStringInner( bool hex ) const {
		wstring result;
		int step;

//		step = this->Optimize();
//		if (step == 0) {
//			return L"";
//		}
		step = 1;

		result.reserve( Resolution / step );
		result.assign( L"" );

		if (hex == true) {
			for (int i = 0; i < Resolution; i += step) {
				result += m_values[i].ToHexString();
			}
		}
		else {
			for (int i = 0; i < Resolution; i += step) {
				result += m_values[i].ToString();
			}
		}

		return result;
	}
}
}
