#include "StdAfx.h"
#include "CsvReader.hpp"
#include "Utility.hpp"

namespace Lmt {
	CsvReader::CsvReader( void ) {
		this->Initialize();
	}

	CsvReader::~CsvReader( void ) {
		this->Close();
	}

	wstring CsvReader::GetValue( void ) const {
		return m_value;
	}

	const wstring & CsvReader::GetValueRef( void ) const {
		return m_value;
		//̃\bh͑oT[rXi
	}

	bool CsvReader::IsOpen( void ) const {
		return (m_pfs != NULL);
	}

	bool CsvReader::Open( const wstring &path, const wchar_t sep ) {
		wfstream *pfs = NULL;

		if (this->IsOpen() == true) {
			return false;
		}
		if (sep == L'\0') {
			return false;
		}

		pfs = new wfstream( Utility::WStringToString( path ).c_str(), ios::in );
		if (pfs == NULL) {
			return false;
		}
		if (pfs->is_open() == false) {
			return false;
		}

		m_pfs = pfs;
		m_sep = sep;
		return true;
	}

	void CsvReader::Close( void ) {
		if (this->IsOpen() == true) {
			m_pfs->close();
			delete m_pfs;
			this->Initialize();
		}
	}

	bool CsvReader::NextColumn( void ) {
		if (this->IsOpen() == false) {
			return false;
		}
		if (m_endl == true) {
			return false;
		}

		size_t nextSep = m_line.find( m_sep, m_next );

		if (nextSep == wstring::npos) {
			nextSep = m_line.length();
			m_endl = true;
		}

		m_index = m_next;
		m_next = nextSep + 1;
		m_value = m_line.substr( m_index, (nextSep - m_index) );

		return true;
	}

	bool CsvReader::Fetch( void ) {
		if (this->IsOpen() == false) {
			return false;
		}
		if (m_pfs->eof() == true) {
			return false;
		}

		wstring line;

		getline( *m_pfs, line );
		if (m_pfs->fail() == true) {
			return false;
		}

		m_index = 0;
		m_next = 0;
		m_line.swap( line );
		m_value = L"";
		m_endl = false;

		return true;
	}

	void CsvReader::Initialize( void ) {
		m_pfs = NULL;
		m_sep = L'\0';
		m_index = 0;
		m_next = 0;
		m_line = L"";
		m_value = L"";
		m_endl = true;
	}
}
