#include "stdafx.h"
#include "Main.hpp"
#include "BmsUtility.hpp"
#include "PtTrackIndexes.hpp"
#include "SoundElement.hpp"
#include "TempoElement.hpp"
#include "Utility.hpp"

int main(int argc, char *argv[] ) {
	locale::global( locale( "jpn", locale::ctype ) );

	return Lmt::Pt2Bms::Main::EntryPoint( argc, argv );
}

using namespace Lmt;
using namespace Lmt::Max2Bms;

namespace Lmt {
namespace Pt2Bms {
	int Main::EntryPoint( int argc, char *argv[] ) {
		Options opts;
		wstring sdbName;
		SongDatabase sdb;
		PatternFile pt;
		BmsGenerator bmsgen;
		PtLoadResult ptResult;
		int result = -1;

		//IvV̉
		if (ParseOptions( argc, argv, opts ) == false) {
			goto quit;
		}

		//yȃf[^x[X̓ǂݍ
		sdbName = Utility::CombinePath( Utility::GetModuleDirectory(), L"songdb.dat" );
		if (sdb.Load( sdbName ) == false) {
			wcerr << L"songdb.dat  Failed to load." << endl;
			goto quit;
		}

		//ΏۂPTt@Cǂݍ
		ptResult = pt.Load( opts.inputPath );
		if (ptResult != PtLoadOk) {
			wcerr << PtLoadResultString( ptResult ) << endl;
			goto quit;
		}

		//wb_[̏
		if (SetupHeader( &pt, &sdb, bmsgen.GetHeader() ) == false) {
			goto quit;
		}

		//BMSfޏ̏
		if (SetupMaterial( &pt, bmsgen.GetMaterial() ) == false) {
			goto quit;
		}

		//V[PX̏
		if (SetupSequence( &pt, sdb.GetKeyCount(), bmsgen.GetSequence() ) == false) {
			goto quit;
		}

		//BMSo
		GenerateBms( &bmsgen );
		result = 0;

quit:
#ifdef _DEBUG
		wcin.get();
#endif
		return result;
	}

	bool Main::ParseOptions( int argc, char *argv[], Options &opts ) {
		if (argc == 1) {
			Usage();
			return false;
		}
		if ((argc == 2) && ((strcmp( argv[1], "-h" ) == 0) || (strcmp( argv[1], "--help" ) == 0))) {
			Usage();
			return false;
		}

		opts.inputPath = Utility::StringToWString( argv[1] );

		return true;
	}

	void Main::Usage( void ) {
		wcerr << L"pt2bms <target-path>" << endl;
		wcerr << L"" << endl;
		wcerr << L"<target-path>  Specified a PT file for DJMAX." << endl;
	}

	wstring Main::PtLoadResultString( PtLoadResult result ) {
		switch (result) {
			case PtLoadOk:
				return L"No error.";
			case PtLoadFailedToOpen:
				return L"Failed to open PT file.";
			case PtLoadIOError:
				return L"PT file is IO error.";
			case PtLoadNotPtFormat:
				return L"Not PT file format.";
			case PtLoadBadSoundEntry:
				return L"Bad sound entry.";
			case PtLoadBadTrack:
				return L"Bad track.";
			default:
				return L"Unknown error.";
		}
	}

	bool Main::SetupHeader( PatternFile *ppt, SongDatabase *psdb, BmsHeader *pheader ) {
		wstringstream ss;
		wstring hash;
		wstring songimg;
		wstring title;

		//ʂ̃nbVlvZ
		hash = ppt->GetHash();

		//ʂ̃nbVŊyDBΏۊyȂƍ
		if (psdb->Select( hash ) == false) {
			//Ώۂ̊yȂˁ[񂾂ǂ
			wcerr << L"[" << hash << L"] Unknown song..." << endl;
			return false;
		}

		//yȂ̃^Cg
		ss.str( L"" );
		ss << psdb->GetTitle();
		ss << L" (";
		ss << psdb->GetKeyCount();
		ss << L"Key-";
		ss << psdb->GetDifficultyName();
		ss << L")";
		title = ss.str();

		//yȂ̃^Cg摜
		ss.str( L"" );
		ss << L"Title";
		ss << psdb->GetDifficultyName();
		ss << L".jpg";
		songimg = ss.str();

		//wb_[̏
		pheader->SetPlayStyle( SinglePlayStyle );
		pheader->SetGenre( psdb->GetGenre() );
		pheader->SetTitle( title );
		pheader->SetArtist( psdb->GetArtist() );
		pheader->SetInitialTempo( psdb->GetTempo() );
		pheader->SetDifficulty( psdb->GetDifficulty() );
		pheader->SetJudgeRanks( EasyJudgeRank );
		pheader->SetTotalGroove( BmsUtility::CalculateTotalGroove( ppt->GetTotalNotes() ) );
		pheader->SetTitleImagePath( songimg );

		return true;
	}

	bool Main::SetupMaterial( PatternFile *ppt, BmsMaterial *pmaterial ) {
		int soundCount;
		SoundEntry sentry;

		//SẴTEh`𒊏o
		soundCount = ppt->GetSoundEntryCount();
		for (int i = 0; i < soundCount; i++) {
			//TEh`o
			if (ppt->GetSoundEntry( i, sentry ) == false) {
				continue;
			}

			//fޏƂēo^
			pmaterial->SetSoundPath( AlphaDecimal( sentry.GetIdentifier() ), sentry.GetFileName() );
		}

		return true;
	}

	bool Main::SetupSequence( PatternFile *ppt, int keyCount, BmsSequence *psequence ) {
		struct PlayableTrack {
			int index;
			BmsChannels linech;
			BmsChannels lnch;
		};

		//ovC\gbN̈ꗗ
		static const PlayableTrack playableTracks5k[] = {
			{ P1Line1TrackIndex, P1Line1BmsCh, P1Ln1BmsCh },
			{ P1Line2TrackIndex, P1Line2BmsCh, P1Ln2BmsCh },
			{ P1Line3TrackIndex, P1Line3BmsCh, P1Ln3BmsCh },
			{ P1Line4TrackIndex, P1Line4BmsCh, P1Ln4BmsCh },
			{ P1Line5TrackIndex, P1Line5BmsCh, P1Ln5BmsCh },
			{ -1 },
		};
		static const PlayableTrack playableTracks7k[] = {
			{ P1ScratchTrackIndex, P1Line1BmsCh, P1Ln1BmsCh },
			{ P1Line1TrackIndex, P1Line2BmsCh, P1Ln2BmsCh },
			{ P1Line2TrackIndex, P1Line3BmsCh, P1Ln3BmsCh },
			{ P1Line3TrackIndex, P1Line4BmsCh, P1Ln4BmsCh },
			{ P1Line4TrackIndex, P1Line5BmsCh, P1Ln5BmsCh },
			{ P1Line5TrackIndex, P1Line6BmsCh, P1Ln6BmsCh },
			{ P1PedalTrackIndex, P1Line7BmsCh, P1Ln7BmsCh },
			{ -1 },
		};

		const PlayableTrack *playableTracks;
		Track track;
		TrackElement element;
		SoundElement *psndElem;
		TempoElement *ptempoElem;
		int elementCount;
		int bkIndex;
		int i;
		int j;
		int position;
		int length;
		int identifier;

		//BMSւ̃L[}bsO
		if (keyCount == 5) {
			playableTracks = playableTracks5k;
		}
		else {
			playableTracks = playableTracks7k;
		}

		//vC\gbN̒o
		for (i = 0; playableTracks[i].index != -1; i++) {
			//gbNo
			if (ppt->GetTrack( playableTracks[i].index, track ) == false) {
				continue;
			}

			//Svf𒊏o
			elementCount = track.GetElementCount();
			for (j = 0; j < elementCount; j++) {
				//vf擾Ǝނ̃`FbN
				if (track.GetElement( j, element ) == false) {
					continue;
				}
				if (element.GetType() != PlaySoundTrackElement) {
					continue;
				}

				//m[g̏𒊏o
				psndElem = static_cast<SoundElement*>( &element );
				position = psndElem->GetPosition();
				length = psndElem->GetLength();
				identifier = psndElem->GetIdentifier();

				//m[g̎ނ𔻒f
				if (length <= 6) {
					//ʏ̃m[g
					psequence->Add( playableTracks[i].linech, position, AlphaDecimal( identifier ) );
				}
				else {
					//Om[g
					psequence->Add( playableTracks[i].lnch, position, AlphaDecimal( identifier ) );
					psequence->Add( playableTracks[i].lnch, position + length, AlphaDecimal( AlphaDecimal::MaxValue ) );
				}
			}
		}

		//e|ύXgbN̒o
		if (ppt->GetTrack( ChangeTempoTrackIndex, track ) == true) {
			elementCount = track.GetElementCount();
			for (j = 0; j < elementCount; j++) {
				//vf擾Ɠo^ۃ`FbN
				if (track.GetElement( j, element ) == false) {
					continue;
				}
				if ((element.GetType() != ChangeTempoTrackElement) || (element.GetPosition() == 0)) {
					continue;
				}

				//o^
				ptempoElem = static_cast<TempoElement*>( &element );
				psequence->Add( ChangeTempoBmsCh, element.GetPosition(), AlphaDecimal( static_cast<int>( ptempoElem->GetTempo() ) ) );
			}
		}

		//obNOEhgbN̒o
		bkIndex = 0;
		for (i = BkBeginTrackIndex; i <= BkEndTrackIndex; i++) {
			//gbNo
			if (ppt->GetTrack( i, track ) == false) {
				continue;
			}

			//obNOEh̃CfbNXݒ
			psequence->SetBackgroundIndex( bkIndex );

			//Svf𒊏o
			elementCount = track.GetElementCount();
			for (j = 0; j < elementCount; j++) {
				//vf擾
				if (track.GetElement( j, element ) == false) {
					continue;
				}

				//vf̎ނo
				switch (element.GetType()) {
					//obNOEhTEh
					case PlaySoundTrackElement:
						psndElem = static_cast<SoundElement*>( &element );
						psequence->Add( BackgroundBmsCh, element.GetPosition(), AlphaDecimal( psndElem->GetIdentifier() ) );
						break;
					//e|ύX
					case ChangeTempoTrackElement:
						ptempoElem = static_cast<TempoElement*>( &element );
						psequence->Add( ChangeTempoBmsCh, element.GetPosition(), AlphaDecimal( static_cast<int>( ptempoElem->GetTempo() ) ) );
						break;
					//ȊO͖
					default:
						break;
				}
			}

			bkIndex++;
		}

		return true;
	}

	void Main::GenerateBms( BmsGenerator *pgenerator ) {
		int lineCount;

		//BMSJn
		pgenerator->Generate();

		//XNvgSCo
		lineCount = pgenerator->GetLineCount();
		for (int i = 0; i < lineCount; i++) {
			wcout << pgenerator->GetLine( i ) << endl;
		}
	}
}
}
