#include "StdAfx.h"
#include "SongDatabase.hpp"
#include "CsvReader.hpp"

namespace Lmt {
namespace Max2Bms {
	SongDatabase::SongDatabase( void ) {
		m_index = -1;
	}

	SongDatabase::~SongDatabase( void ) {
		this->Clear();
	}

	wstring SongDatabase::GetHash( void ) const {
		return m_items[m_index].hash;
	}

	int SongDatabase::GetIdentifier( void ) const {
		return m_items[m_index].identifier;
	}

	wstring SongDatabase::GetGenre( void ) const {
		return m_items[m_index].genre;
	}

	wstring SongDatabase::GetTitle( void ) const {
		return m_items[m_index].title;
	}

	wstring SongDatabase::GetArtist( void ) const {
		return m_items[m_index].artist;
	}

	int SongDatabase::GetTempo( void ) const {
		return m_items[m_index].tempo;
	}

	int SongDatabase::GetKeyCount( void ) const {
		return m_items[m_index].keyCount;
	}

	wstring SongDatabase::GetDifficultyName( void ) const {
		return m_items[m_index].difficultyName;
	}

	int SongDatabase::GetDifficulty( void ) const {
		return m_items[m_index].difficulty;
	}

	void SongDatabase::Clear( void ) {
		m_items.clear();
		m_indexMap.clear();
		m_index = -1;
	}

	bool SongDatabase::Load( const wstring &path ) {
		CsvReader reader;
		SongItem item;
		wstring tempStr;
		int tempInt;
		int index = 0;

		if (reader.Open( path, L'\t' ) == false) {
			return false;
		}

		this->Clear();

		while (reader.Fetch() == true) {
			//hash
			reader.NextColumn();
			tempStr = reader.GetValue();
			if (tempStr.length() != 32) {
				continue;
			}
			item.hash = tempStr;

			//identifier
			reader.NextColumn();
			tempStr = reader.GetValue();
			tempInt = wcstol( tempStr.c_str(), NULL, 10 );
			if ((tempInt <= 0) || (tempInt > 999)) {
				continue;
			}
			item.identifier = tempInt;

			//genre
			reader.NextColumn();
			tempStr = reader.GetValue();
			if (tempStr.length() == 0) {
				continue;
			}
			item.genre = tempStr;

			//title
			reader.NextColumn();
			tempStr = reader.GetValue();
			if (tempStr.length() == 0) {
				continue;
			}
			item.title = tempStr;

			//artist
			reader.NextColumn();
			tempStr = reader.GetValue();
			if (tempStr.length() == 0) {
				continue;
			}
			item.artist = tempStr;

			//tempo
			reader.NextColumn();
			tempStr = reader.GetValue();
			tempInt = wcstol( tempStr.c_str(), NULL, 10 );
			if ((tempInt <= 0) || (tempInt > 255)) {
				continue;
			}
			item.tempo = tempInt;

			//keyCount
			reader.NextColumn();
			tempStr = reader.GetValue();
			tempInt = wcstol( tempStr.c_str(), NULL, 10 );
			if ((tempInt != 5) && (tempInt != 7)) {
				continue;
			}
			item.keyCount = tempInt;

			//difficultyName
			reader.NextColumn();
			tempStr = reader.GetValue();
			if ((tempStr != L"Easy") && (tempStr != L"Normal") &&
				(tempStr != L"Hard") && (tempStr != L"MX") &&
				(tempStr != L"SC")) {
				continue;
			}
			item.difficultyName = tempStr;

			//difficulty
			reader.NextColumn();
			tempStr = reader.GetValue();
			tempInt = wcstol( tempStr.c_str(), NULL, 10 );
			if ((tempInt < 1) || (tempInt > 99)) {
				continue;
			}
			item.difficulty = tempInt;

			if (m_indexMap.insert( make_pair( item.hash, index ) ).second == false) {
				continue;
			}

			m_items.push_back( item );
			index++;
		}

		return true;
	}

	bool SongDatabase::Select( const wstring &hash ) {
		IndexMap::iterator it = m_indexMap.find( hash );

		if (it == m_indexMap.end()) {
			return false;
		}

		m_index = it->second;

		return true;
	}
}
}
